@@ -7,6 +7,11 @@ from collections import defaultdict, Counter
 import pandas as pd
 import signal
 import numpy as np
+from shutil import copyfile
+import pyfaidx
+from random import choice
+from pyliftover import LiftOver
+from Bio.Seq import reverse_complement
 
 from mutyper import ancestor
 
@@ -22,6 +27,64 @@ def setup_ancestor(args):
                              sequence_always_upper=(not args.strict))
 
 
+def ancestral_fasta(args):
+    """subroutine for ancestor subcommand
+    """
+    # single chromosome fasta file for reference genome
+    ref = pyfaidx.Fasta(args.reference, read_ahead=10000)
+    # make a copy to build our ancestor for this chromosome
+    copyfile(args.reference, args.output)
+    anc = pyfaidx.Fasta(args.output, read_ahead=10000, mutable=True)
+    # reference genome for outgroup species (all chromosomes)
+    out = pyfaidx.Fasta(args.outgroup, read_ahead=10000)
+    # outgroup to reference alignment chain file
+    lo = LiftOver(args.chain)
+    # snps database for the same chromosome
+    vcf = cyvcf2.VCF(args.vcf)
+
+    # change regions outside of callability mask to all N bases
+    if args.bed:
+        if args.bed == '-':
+            bed = sys.stdin
+        else:
+            bed = open(args.bed, 'r')
+        last_end = 0
+        for line in bed:
+            chrom, start, end = line.rstrip().split('\t')[:3]
+            start = int(start)
+            anc[chrom][last_end:start] = 'N' * (start - last_end)
+            last_end = int(end)
+        anc[chrom][last_end:
+                   len(anc[chrom])] = 'N' * (len(anc[chrom]) - last_end)
+
+    for variant in vcf:
+        # change variants that are not biallelic SNPs to N bases
+        if not (variant.is_snp and len(variant.ALT) == 1):
+            anc[chrom][variant.start:
+                       variant.end] = 'N' * (variant.end - variant.start)
+        else:
+            out_coords = lo.convert_coordinate(variant.CHROM, variant.start)
+            # change ambiguously aligning sites to N bases
+            if out_coords is None or len(out_coords) != 1:
+                anc[chrom][variant.start] = 'N'
+            else:
+                if variant.REF != ref[chrom][variant.start].seq.upper():
+                    raise ValueError(f'variant reference allele {variant.REF} '
+                                     f'mismatches reference sequence '
+                                     f'{ref[chrom][variant.start]}')
+                out_chromosome, out_position, out_strand = out_coords[0][:3]
+                out_allele = out[out_chromosome][out_position].seq
+                # if negative strand, take reverse complement base
+                if out_strand == '-':
+                    out_allele = reverse_complement(out_allele)
+                # and finally, polarize
+                if out_allele.upper() == variant.ALT[0]:
+                    anc[chrom][variant.start] = out_allele
+                elif out_allele.upper() != variant.REF:
+                    # triallelic
+                    anc[chrom][variant.start] = 'N'
+
+
 def variants(args):
     """subroutine for variants subcommand
     """
@@ -103,8 +166,14 @@ def spectra(args):
         spectra_data = defaultdict(lambda: np.zeros_like(vcf.samples,
                                                          dtype=int))
 
-        for variant in vcf:
-            spectra_data[variant.INFO['mutation_type']] += variant.gt_types
+        if args.randomize:
+            for variant in vcf:
+            	random_haplotype = choice([x for x, y in enumerate(variant.gt_types) 
+            						 for _ in range(y)])
+            	spectra_data[variant.INFO['mutation_type']][random_haplotype] += 1.
+        else:
+            for variant in vcf:
+                spectra_data[variant.INFO['mutation_type']] += variant.gt_types
 
         spectra = pd.DataFrame(spectra_data,
                                vcf.samples).reindex(sorted(spectra_data),
@@ -151,6 +220,10 @@ def get_parser():
     subparsers = parser.add_subparsers(
         title='subcommands', description='specify one of these', required=True,
         help='additional help available for each subcommand')
+    parser_ancestor = subparsers.add_parser(
+        'ancestor', description='create an ancestral FASTA file, using an '
+                                'outgroup alignment, and a database of SNPs '
+                                'with a callability mask')
     parser_variants = subparsers.add_parser(
         'variants', description='adds mutation_type to VCF/BCF INFO, polarizes'
                                 ' REF/ALT/AC according to ancestral/derived '
@@ -180,9 +253,9 @@ def get_parser():
         sub_parser.add_argument('--sep', type=str, default=':',
                                 help='field delimiter in FASTA headers '
                                 '(default ":")')
-        sub_parser.add_argument('--chrom_pos', type=int, default=2,
+        sub_parser.add_argument('--chrom_pos', type=int, default=0,
                                 help='0-based chromosome field position in '
-                                'FASTA headers (default 2)')
+                                'FASTA headers (default 0)')
         sub_parser.add_argument('--strand_file', type=str, default=None,
                                 help='path to bed file with regions where '
                                      'reverse strand defines mutation '
@@ -194,23 +267,43 @@ def get_parser():
                                      'considered ancestrally identified')
 
     # subcommands that require VCF input
-    for sub_parser in (parser_variants, parser_spectra, parser_ksfs):
+    for sub_parser in (parser_ancestor, parser_variants, parser_spectra,
+                       parser_ksfs):
         sub_parser.add_argument('vcf', type=str,
-                                help='VCF/BCF file created by the variants '
-                                     'subcommand ("-" for stdin)')
+                                help='VCF/BCF file, usually for a single '
+                                     'chromosome ("-" for stdin)')
+
+    # subcommands that take BED input
+    for sub_parser in (parser_ancestor, parser_targets):
+        sub_parser.add_argument('--bed', type=str, default=None,
+                                help='path to BED file mask ("-" for stdin)')
+
+    # arguments specific to ancestor subcommand
+    parser_ancestor.add_argument('reference', type=str,
+                                 help='path to reference FASTA for one '
+                                      'chromosome')
+    parser_ancestor.add_argument('outgroup', type=str,
+                                 help='path to outgroup genome FASTA')
+    parser_ancestor.add_argument('chain', type=str,
+                                 help='path to alignment chain file '
+                                      '(reference to outgroup)')
+    parser_ancestor.add_argument('output', type=str,
+                                 help='path for output ancestral FASTA for '
+                                      'this chromosome')
+    parser_ancestor.set_defaults(func=ancestral_fasta)
 
     # arguments specific to variants subcommand
     parser_variants.set_defaults(func=variants)
 
     # arguments specific to targets subcommand
-    parser_targets.add_argument('--bed', type=str, default=None,
-                                help='path to BED file mask ("-" for stdin)')
     parser_targets.set_defaults(func=targets)
 
     # arguments specific to spectra subcommand
     parser_spectra.add_argument('--population', action='store_true',
                                 help='population-wise spectrum, instead of '
                                      'individual')
+    parser_spectra.add_argument('--randomize', action='store_true',
+    							help='randomly assign mutation to a single haplotype')
     parser_spectra.set_defaults(func=spectra)
 
     # arguments specific to ksfs subcommand
